#!/m1/shared/perl/5.8.8_Oracle10/bin/perl -w

###############################################################################
#                                                                             #
#  New Books Subject Feed, Version 1.0 for Unix                               #
#                                                                             #
#  new_books_subjects.cgi for 'vwebv' version of WebVoyage                    #
#                                                                             #
#  Based on scripts by Michael Doran, doran@uta.edu                           #
#  University of Texas at Arlington Libraries                                 #
#  Box 19497, Arlington, TX 76019, USA                                        #
#  More information about newBooks at: http://rocky.uta.edu/doran/newbooks/   #
#                                                                             #
#  Edited by Jim Robinson, jim.robinson@tccd.edu 2009-06-18                   #
#  Questions or comments about this script can be sent to Jim Robinson        #
#  jim.robinson@tccd.edu                                                      #
#                                                                             #
###############################################################################
#                                                                             #
#  Copyright 2000-2008, The University of Texas at Arlington ("UTA").         #
#  All rights reserved.  See included LICENSE for particulars.                #
#                                                                             #
###############################################################################
#                                                                             #
#  Call the script with as many LOC classification letters/numbers as desired #
#  in the following format where:                                             #
#                                                                             #
#  * U and V are the first letters in LOC call numbers                        #
#  * title is the name of the subject (optional, only the last will be used)  #
#  * sort is the field to sort on as determined in the getSortOrder function  #
#  * sort is optional and will default to the item title                      #
#                                                                             #
#  http://path.to.this.script?subject1=U&subject2=V&title=Military+Science.+Naval+Science&sort=display_call_number
#                                                                             #
###############################################################################

use warnings;
use strict;
use Encode;
use Unicode::Normalize;

################################################################################
# Configuration - Stuff you can change relatively safely                       #
################################################################################
my $library_domain   = "library.tccd.edu";

my $feed_title 		= "Tarrant County College Libraries New Books";
my $feed_description = "New titles available at TCC libraries";

my $logo_image_url 	= "http://$library_domain/vwebv/ui/tcc_images/tcc_logo_2008.jpg";
my $logo_image_title = "Tarrant County College District Logo";

my $default_sort     = "title";

# probably don't need to change these last 3 variables...
my $holdingsInfo_url = "http://$library_domain/vwebv/holdingsInfo?bibId=";
my $this_feed_url 	= "http://$library_domain/vwebv/new_books_subjects.cgi";
my $newBooks_url 		= "http://$library_domain/vwebv/newBooks.cgi?sk=en_US";
################################################################################
# End Configuration - Stuff you can change relatively safely                   #
################################################################################



################################################################################
# Get Input Parameters                                                         #
################################################################################
my %params = {};
getParams();
my $subject = $params{'title'} ne "" ? "in " . $params{'title'} : "";
my $sort = $params{'sort'} ne "" ? lc($params{'sort'}) : $default_sort;

my $pub_date;
getTime();

print "Content-Type: application/rss+xml; charset=utf-8\n\n";
my $feed;
getFeederHead();

################################################################################
# Read data file                                                               #
################################################################################
my @data;
my $file = "../../newbooks/newBooks.txt";
open(DATA,$file) || die "Can't open $file: $!\n";
while(<DATA>) {
	push @data, [ split(/\t/) ];
}
close(DATA);

################################################################################
# Sort data                                                                    #
################################################################################
my @sorted_data;
my $sort_index = &getSortOrder();
@sorted_data = $sort_index ne "" ? sort { $a->[$sort_index] cmp $b->[$sort_index] } @data : @data;

################################################################################
# Process data / build RSS content                                             #
################################################################################
for my $tmp (@sorted_data) {
	my ($bib_id, $isbn, $bib_format, $author, $title, $edition, $imprint, $location, $location_temp, $dispcallno, $callno, $week) = @$tmp;
	
	while((my $key,my $value) = each %params) {
		if($key =~ /^subject/) {
			if($dispcallno =~ /^$value/i) {
				$bib_id        = decode('UTF-8', $bib_id);
				$author        = decode('UTF-8', $author);
				$title         = decode('UTF-8', $title);
				$location      = decode('UTF-8', $location);
 	      
				# Normalize: Unicode Normalization Form D (canonical decomposition) 
				# i.e. decompose any precomposed accented characters
				$title  = "         <title>".NFD($title)."</title>\n";
	
				my $description = "         <description>".$location.", ".$dispcallno."</description>\n";
				my $link        = "         <link>$holdingsInfo_url$bib_id</link>\n";
				my $pub         = "         <pubDate>$pub_date</pubDate>\n";
				my $guid        = "         <guid>$holdingsInfo_url$bib_id</guid>\n";
				my $category	 = "         <category>".$location.", ".$dispcallno."</category>\n";
	
				$feed .= "      <item>\n$title.$description.$pub.$guid.$link.$category      </item>\n";
				$feed =~ s/&/and/g; #Converts ampersands to 'and' because XML doesn't like unencoded ampersands!
			}
		}
	}
}

################################################################################
# Finish and print                                                             #
################################################################################	
$feed .= "   </channel>\n</rss>";
print $feed;



################################################################################
# Subroutines                                                                  #
################################################################################

################################################################################
# Formats the time for an RSS feed                                             #
################################################################################
sub getTime() {
	my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime (time);
	my @Days = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat");
	my @Months = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
	my $day = $Days[$wday];
	my $date = $mday + 0;
	my $year = 1900 + $year;
	my $month = $Months[$mon];
	$pub_date = "$day, $date $month $year 03:00:00 -0006";
}

################################################################################
# Get parameters passed by the user and load them in a hash                    #
################################################################################
sub getParams() {
	my @values = split(/&/,$ENV{'QUERY_STRING'});
	foreach my $i(@values) {
		(my $key, my $value)=split(/=/,$i);
		$value =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$params{$key}=$value;
	}
}

################################################################################
# Create the RSS feed header                                                   #
################################################################################
sub getFeederHead() {
	$feed = "<?xml version=\"1.0\"?>
<rss version=\"2.0\" xmlns:atom=\"http://www.w3.org/2005/Atom\">
	<channel>
      <title>$feed_title</title>
      <link>$newBooks_url</link>
      <description>$feed_description $subject</description>
      <language>en-us</language>
      <atom:link href='$this_feed_url' rel='self' type='application/rss+xml' />
";
	if($logo_image_url) {
		$feed .= "
   	<image>
     	 	<url>$logo_image_url</url>
     	 	<title>$logo_image_title</title>
     	 	<link>$newBooks_url</link>
     	 </image>
		";
	}
}

################################################################################
# Determine the sort index based on user input. Defaults to title (4)          #
#                                                                              #
# The following array lists possible terms users might use to sort on.         #
# Use lower case when adding more.                                             #
# bib_id=0, isbn=1, bib_format=2, author=3, title=4, edition=5, imprint=6      #
# location=7, location_temp=8, dispcallno=9, callno=10, week=11                #
################################################################################
sub getSortOrder() {
	my %sort_order = (
		"bib_id"              =>"0",
		"bibid"               =>"0",
		
		"isbn"                =>"1",
		
		"bib_format"          =>"2",
		"bibformat"           =>"2",
		"format"              =>"2",
		
		"author"              =>"3",
		"title"               =>"4",
		"edition"             =>"5",
		"imprint"             =>"6",
		
		"location"            =>"7",
		"perm_location"       =>"7",
		"location_perm"       =>"7",
		"permanent_location"  =>"7",
		
		"location_temp"       =>"8",
		"temp_location"       =>"8",
		"temporary_location"  =>"8",
		
		"dispcallno"          =>"9",
		"display_call_no"     =>"9",
		"display_call_number" =>"9",
		"display_call"        =>"9",
		
		"call_no"             =>"10",
		"call_num"            =>"10",
		"call_number"         =>"10",
		"call"                =>"10",
		
		"week"                =>"11"
	);
	my $sindex = $sort_order{$sort} ne "" ? $sort_order{$sort} : "";
	return $sindex;
}

exit(0);